/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
package de.bsvrz.vew.sim.sim.datengenerator;

import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveDataQueryResult;
import de.bsvrz.dav.daf.main.archive.ArchiveDataSpecification;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestManager;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.ArchiveTimeSpecification;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Ein Adapter, der zwischen einen Stream vom Archivsystem und einem Protokollierungsmodul steht und
 * die von der take() Methode kommenden Daten umwandelt und mit ihnen die update() Methode aufruft.
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.3 $ / $Date: 2008/12/03 14:00:32 $ / ($Author: Sans $)
 */
public class ArchivAdapter extends Thread 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Max. Anzahl-Verbindungsversuche, wenn zu Beginn keine Verbindung zum Archiv besteht
   */
  public final static int ANZAHL_VERBINDUNGS_VERSUCHE = 5;
    
  /**
  * die Datenspezifikationen, fr welche die Archivanfrage gestellt wird
  */
  private Collection<DatenSpezifikation> _datenSpezifikationen;
  
  /**
   * Die Datengenrator-Instanz, die die Archivdaten als Eingangsdaten bentigt
   */
  private DatenGeneratorOfflineSimulation _generator;

  /**
   * Erzeugt eine neue Instanz vom Typ ArchivAdapter
   * @param datenSpezifikationen die Datenspezifikationen, fr welche die Archivanfrage gestellt wird
   * @param generator die Datengenrator-Instanz, die die Archivdaten als Eingangsdaten bentigt
   */
  public ArchivAdapter(Collection<DatenSpezifikation> datenSpezifikationen,
      DatenGeneratorOfflineSimulation generator)
  {
    _datenSpezifikationen = datenSpezifikationen;
    _generator = generator;
  }
 
  /**
   * Einstiegspunkt fr die Archivanfrage
   * 
   * @see java.lang.Runnable#run()
   */
  @Override
  public void run() 
  {
    // /// Einstellungen aus Eingabe in Objekte umwandeln
    _debug.config("Archivadapter gestartet");
    
    ArchiveQueryPriority archiveQueryPriority = ArchiveQueryPriority.MEDIUM;
    ArchiveTimeSpecification archiveTimeSpecification = new ArchiveTimeSpecification(TimingType.ARCHIVE_TIME,
        false, _generator.getSimulationsBeginn(), _generator.getSimulationsEnde());
    ArchiveDataKindCombination archiveDataKindCombination = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE, 
        ArchiveDataKind.ONLINE_DELAYED, ArchiveDataKind.REQUESTED, ArchiveDataKind.REQUESTED_DELAYED);
    ArchiveOrder archiveOrder = ArchiveOrder.BY_DATA_TIME;
    ArchiveRequestOption archiveRequestOption = ArchiveRequestOption.NORMAL;
    
    ArchiveRequestManager archive = _generator.getConnection().getArchive();
    List<ArchiveDataStream[]> streamArrayList = new LinkedList<ArchiveDataStream[]>();
    int n = 0;
    try
    {
      _debug.info("Start Archivanfrage");
    
      int anzahl = 0;
      while(!archive.isArchiveAvailable()) 
      {
        if (!_generator.isGestartet())
        {
          return;
        }
        sleep(1000);
                       
        Debug.getLogger().fine("Warte auf das Archivsystem: Versuch " + anzahl + " von " + ANZAHL_VERBINDUNGS_VERSUCHE);
        if (anzahl >= ANZAHL_VERBINDUNGS_VERSUCHE)
        {  
          Debug.getLogger().error("Es konnte keine Verbindung zum Archivsystem aufgebaut werden");
          return;
        }
        anzahl++;
      }
                   
      for(DatenSpezifikation ds : _datenSpezifikationen)
      {
        List<ArchiveDataSpecification> archiveDataSpecifications = new LinkedList<ArchiveDataSpecification>(); 
        Collection<SystemObject> sysObjects = ds.getSystemObjekte();
        //DataDescription dataDescription = new DataDescription(ds.getAttributeGroup(), 
        //    ds.getAspect(), ds.getSimulationVariant());
       
        for (Iterator iterator = sysObjects.iterator(); _generator.isGestartet() && iterator.hasNext();) 
        {
            SystemObject sysObject = (SystemObject) iterator.next();
            archiveDataSpecifications.add(new ArchiveDataSpecification(archiveTimeSpecification, 
                archiveDataKindCombination, archiveOrder, archiveRequestOption, /*dataDescription*/ds, sysObject));
        }
        if (!_generator.isGestartet())
        {
          return;
        }
        
        _debug.config("Erzeuge Archivanfrage fr " + ds);
        ArchiveDataQueryResult queryResult = archive.request(archiveQueryPriority, archiveDataSpecifications);
        
        if (queryResult.isRequestSuccessful()) 
        {
          streamArrayList.add(queryResult.getStreams());
        }
        else
        {
          _debug.error("Archivanfrage fr " + ds + " konnte nicht bearbeitet werden:\n" + queryResult.getErrorMessage());
        }
      }
      
      _debug.config("Start Daten abholen von Archiv fr Simulation " + _generator.getSimulation());
      
      /*
      boolean done = false;
      
      while (_generator.isGestartet() && !done)
      {
        done = true;
        Iterator<ArchiveDataStream[]> it = streamArrayList.iterator();
        while(_generator.isGestartet() && it.hasNext())
        {
          ArchiveDataStream[] streamArray = it.next();
          for (int i = 0; _generator.isGestartet() && i <  streamArray.length; i += 1)
          {
            ArchiveData ad = streamArray[i].take();
            if(ad != null)
            {
              if (ad.getDataType()!= DataState.END_OF_ARCHIVE)
              {  
                ResultData rd = new ResultData(ad.getObject(), 
                    ad.getDataDescription(), ad.getDataTime(), ad.getData());
                _generator.getQueue().put(rd);
              }
              done = false;
            }
          }
        }  
      }
      */
      
      while (_generator.isGestartet() && !streamArrayList.isEmpty())
      {
        ListIterator<ArchiveDataStream[]> it = streamArrayList.listIterator();
        while(_generator.isGestartet() && it.hasNext())
        {
          ArchiveDataStream[] streamArray = it.next();
          boolean leer = true;
          for (int i = 0; _generator.isGestartet() && i <  streamArray.length; i += 1)
          {
            if (streamArray[i] != null)
            {
              ArchiveData ad = streamArray[i].take();
              if(ad != null)
              {
                if (ad.getDataType()!= DataState.END_OF_ARCHIVE)
                {  
//                  _generator.putData(ad.getObject(), ad.getDataDescription().getAttributeGroup(),
//                      ad.getDataDescription().getAspect(), ad.getDataTime(), ad.getData());
                  _generator.putData(ad);
                  n++;
                }
                leer = false;
              }
              else
              {
                streamArray[i].abort();
                streamArray[i] = null;
              }
            }  
          }
          if (leer)
          {
            it.remove();
          }  
        }  
      }
    }  
    catch(InterruptedException e)
    {
      _debug.error("Ein Vorgang wurde unerwartet abgebrochen", e);
    }
    catch(IOException e)
    {
      _debug.error("Fehler bei Archivanfrage", e);
    }
    finally
    {
      for (ArchiveDataStream[] streamArray: streamArrayList)
      {
        for (ArchiveDataStream stream: streamArray)
        {
          if (stream != null)
          {  
            stream.abort();
          }  
        }
      }
      _debug.info("Archivanfrage terminiert");
    }
  }
  
}
