/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.sim.sim.simulation;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.sim.sim.simulationsstrecke.SimulationsStrecke;
import de.bsvrz.vew.sim.sim.steuerung.SteuerungOnlineSimulation;
import de.bsvrz.vew.sim.sim.steuerung.SteuerungSimulation;

/**
 * Modelliert die Online-Simulation als Empfnger -und Sender von 
 * Verwaltungsinformationen und als endlicher Automat
 *
 * @version $Revision: 1.3 $ / $Date: 2008/10/31 13:30:30 $ / ($Author: Sans $)
 *
 * @author Dambach-Werke GmbH
 * @author Stefan Sans
 *
 */
public class OnlineSimulation extends Simulation
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Die Steuerung fr diese Simulation
   */
  private SteuerungSimulation _steuerung;
  
  /**
   * Der endliche Automat fr diese Simulation
   */
  private EndlicherAutomat _endlicherAutomat; 
  
  /**
   * Erzeugt eine neue Instanz vom Typ OnlineSimulation
   *  
   * @param obj das dynamische Simulationsobjekt
   * @param sv die Simulationsvariante
   * @param ss die Simulationsstrecke (Klasse Simulationsstrecke)
   * @param con die Datenverteiler-Verbindung
   * @param anfangsZustand der Anfangszustand 
   */
  public OnlineSimulation(SystemObject obj, short sv, SimulationsStrecke ss,
      ClientDavInterface con, ZustandsBezeichner anfangsZustand)
  {
    super(obj, sv, ss, con);
    
    Zustand neu = new Zustand(ZustandsBezeichner.neu.text);
    Zustand vorstart = new Zustand(ZustandsBezeichner.Vorstart.text);
    Zustand start = new Zustand(ZustandsBezeichner.Start.text);
    Zustand stop = new Zustand(ZustandsBezeichner.Stop.text);
    Zustand geloescht = new Zustand(ZustandsBezeichner.geloescht.text);
    
    _endlicherAutomat = new EndlicherAutomat(5);
    
    _endlicherAutomat.hinzufuegenZustand(neu);
    _endlicherAutomat.hinzufuegenZustand(vorstart);
    _endlicherAutomat.hinzufuegenZustand(start);
    _endlicherAutomat.hinzufuegenZustand(stop);
    _endlicherAutomat.hinzufuegenZustand(geloescht);
    
    if (anfangsZustand.id < _endlicherAutomat.getAnzahlZustaende())
    {  
      Zustand initialZustand = _endlicherAutomat.getZustand(anfangsZustand.id);
      _endlicherAutomat.setZustand(initialZustand);
    }
    else
    {
      throw new IllegalStateException("Der Zustand " + anfangsZustand + 
          " ist fr diesen Typ von Simulation nicht definiert");
    }
    
    _endlicherAutomat.hinzufuegenUebergang(neu,      0, vorstart);
    _endlicherAutomat.hinzufuegenUebergang(vorstart, 1, start);
    _endlicherAutomat.hinzufuegenUebergang(start,    2, stop);
    _endlicherAutomat.hinzufuegenUebergang(stop,     3, geloescht);
    _endlicherAutomat.hinzufuegenUebergang(stop,     4, vorstart);
      
    _steuerung = new SteuerungOnlineSimulation(this);
  }

  /*
  public OnlineSimulation(SystemObject obj, short sv, SimulationsStrecke ss,
      ClientDavInterface con)
  {
    this(obj, sv, ss, con, ZustandsBezeichner.neu);
  }
  */  
     
  
  @Override
  public SteuerungSimulation getSteuerung()
  {
    return _steuerung;
  }
   
  @Override
  protected EndlicherAutomat getEndlicherAutomat()
  {
    return _endlicherAutomat;
  }
  
  @Override
  public void anmelden()
  {
    super.anmelden();
    
    if (getObjekt().isOfType("typ.onlineSimulation"))
    {
      ReceiverRole empfaengerrolle = ReceiverRole.drain();
      ReceiveOptions anmeldeoptionen = ReceiveOptions.normal();
      Aspect aspekt = getObjekt().getDataModel().getAspect("asp.vorgabe");  
      AttributeGroup attributgruppe = getObjekt().getDataModel().getAttributeGroup("atg.simulationsSteuerungOnline");
      short simulationsVariante = getVarianteFuerSimulation();
      DataDescription datenbeschreibung = new DataDescription(attributgruppe, aspekt, simulationsVariante);
      getConnection().subscribeReceiver(this, getObjekt(), datenbeschreibung, anmeldeoptionen, empfaengerrolle);
    }
  }
  
  @Override
  public void abmelden()
  {
    super.abmelden();
    
    if (getObjekt().isOfType("typ.onlineSimulation"))
    {
      short simulationsVariante = getVarianteFuerSimulation();
      Aspect aspekt = getObjekt().getDataModel().getAspect("asp.vorgabe");
      AttributeGroup attributgruppe = getObjekt().getDataModel().getAttributeGroup("atg.simulationsSteuerungOnline");
      DataDescription datenbeschreibung = new DataDescription(attributgruppe, aspekt, simulationsVariante);
      getConnection().unsubscribeReceiver(this, getObjekt(), datenbeschreibung);
    }
  }
    
  @Override
  public void update(ResultData[] resultData)
  {
    super.update(resultData);
    
    for (ResultData data : resultData)
    {
      if (data.getDataState() == DataState.DATA)
      {
        Aspect asp = data.getDataDescription().getAspect();
        AttributeGroup atg = data.getDataDescription().getAttributeGroup();
        short sv = data.getDataDescription().getSimulationVariant();
        
        if (sv == getVarianteFuerSimulation() && asp.getPid().equals("asp.vorgabe"))
        {
          if (atg.getPid().equals("atg.simulationsSteuerungOnline"))
          {
            _debug.fine("Bearbeite Daten mit atg/asp/sv: " + atg.getPid() + "/" +
                asp.getPid() + "/" + sv + ":\n" + data);
            
            Data datenSatz = data.getData();
            TextValue textValue = datenSatz.getTextValue("SimulationsZustand");
            starteZustandswechsel(textValue.getText());
          }  
        }
      }
    }  
  }
}
